import React, { useCallback, useState, useEffect, useImperativeHandle } from 'react';
import { stringify } from 'querystring';
import { notification } from 'antd';
import request from '@/utils/request'
import TableList from './TableList';
import Pagination from './Pagination';

export default React.forwardRef((props, ref) => {
  const [loading, setLoading] = useState(false)
  const [pageConfig, setPageConfig] = useState({ current: 1, pageSize: 10, total: 0 })
  const [list, setList] = useState()
  const { api, params = {}, onPageChange, ...tableProps } = props
  const paramsStr = stringify({
    ...params,
    pageNum: pageConfig.current,
    pageSize: pageConfig.pageSize,
  })

  const updatePageConfig = useCallback((config) => {
    const newPage = {
      ...pageConfig,
      ...config,
    }

    setPageConfig(newPage)

    if (onPageChange) {
      onPageChange(newPage)
    }
  }, [pageConfig])

  const handlePageChange = (current, pageSize) => {
    const config = pageSize ? { current, pageSize } : { current }
    updatePageConfig(config)
  }

  // 获取远程数据
  const fetchData = useCallback(
    () => {
      const url = `${api.url}?${paramsStr}`
      setLoading(true)
      request({...api, url}).then((res) => {
        const {current, total, records = []} = res
        setLoading(false)
        setList(records)
        updatePageConfig({current, total})
      }).catch((err) => {
        console.error(err)
        setLoading(false)
        notification.warn({ message: err.message })
      })
    },
    [paramsStr, api],
  )

  // 监控查询参数的修改
  useEffect(() => {
    fetchData()
  }, [paramsStr])

  // 透传给父组件部分方法
  useImperativeHandle(ref, () => ({
    reload: fetchData
  }))

  return (
    <>
      <TableList loading={loading} dataSource={list} {...tableProps} />
      <Pagination {...pageConfig} onChange={handlePageChange} />
    </>
  )
});
